# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

import logging

from amira.data_processor import OSXCollectorDataProcessor
from amira.s3 import S3Handler
from amira.sqs import SqsHandler


class AMIRA(object):
    """Runs the automated analysis based on the new elements in an S3
    bucket:
        1. Receives the messages from the SQS queue about the new
           objects in the S3 bucket.
        2. Retrieves the objects (OSXCollector output files) from the
           bucket.
        3. Runs the Analayze Filter on the retrieved OSXCollector
           output.
        4. Uploads the analysis results.

    JIRA integration is optional. If any of the JIRA parameters
    (`jira_server`, `jira_user`, `jira_password` or `jira_project`)
    is not supplied or `None`, attaching the analysis results to a JIRA
    issue will be skipped.

    :param region_name: The AWS region name where the SQS queue
                        containing the S3 event notifications is
                        configured.
    :type region_name: string
    :param queue_name: The name of the SQS queue containing the S3
                       event notifications.
    :type queue_name: string
    """

    def __init__(self, region_name, queue_name):
        self._sqs_handler = SqsHandler(region_name, queue_name)
        self._s3_handler = S3Handler()
        self._results_uploader = []
        self._data_feeds = {}
        self._data_processor = OSXCollectorDataProcessor()

    def register_results_uploader(self, results_uploader):
        """Registers results uploader.

        Results uploader will upload the analysis results and the
        summary to a specific destination after the analysis is
        finished.
        """
        self._results_uploader.append(results_uploader)

    def register_data_feed(self, feed_name, generator):
        """Register data input which to be used by the OsXCollector filters

        :param feed_name: Name of the data feed
        :param generator: Generator function providing the data
        """
        self._data_feeds[feed_name] = generator

    def register_data_processor(self, processor):
        """Registers DataProcessor object to process and analyze input data from S3.
        If no processor is registered Amira will fall back using the default
        OSXCollector result processor.

        :param processor: DataProcessor object instance
        """
        self._data_processor = processor

    def run(self):
        """Fetches the OSXCollector output from an S3 bucket based on
        the S3 ObjectCreated event notifications and runs the Analyze
        Filter on the output file.
        Once the analysis is finished the output and the "very readable
        output" files are uploaded to the target S3 bucket.
        """
        created_objects = self._sqs_handler.get_created_objects()

        for created_object in created_objects:
            if created_object.key_name.endswith('.tar.gz'):
                self._process_created_object(created_object)
            else:
                logging.warning(
                    'S3 object {0} name should end with ".tar.gz"'
                    .format(created_object.key_name),
                )

    def _process_created_object(self, created_object):
        """Fetches the object from an S3 bucket and runs the analysis.
        Then it sends the results to the target S3 bucket and attaches
        them to the JIRA ticket.
        """
        # fetch forensic data from the S3 bucket
        forensic_output = self._s3_handler.get_contents_as_string(
            created_object.bucket_name, created_object.key_name,
        )
        processed_input = self._data_processor.process_input(forensic_output)

        try:
            self._data_processor.perform_analysis(processed_input)
        except Exception as exc:
            # Log the exception and do not try any recovery.
            # The message that caused the exception will be deleted from the
            # SQS queue to prevent the same exception from happening in the
            # future.
            logging.warning(
                'Unexpected error while running the Analyze Filter for the '
                'object {}: {}'.format(created_object.key_name, exc),
            )
        try:
            self._data_processor.upload_results(
                created_object.key_name[:-7], self._results_uploader,
            )
        except Exception:
            logging.exception(
                'Unexpected error while uploading results for the '
                'object: {0}'.format(created_object.key_name),
            )



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

from collections import namedtuple


FileMetaInfo = namedtuple('FileMetaInfo', ['name', 'content', 'content_type'])


class ResultsUploader(object):

    """Parent class for the AMIRA results uploaders. Results uploaders
    should expose a single method, ``upload_results()``, that should
    take a list of ``FileMetaInfo`` tuples.
    """

    def upload_results(self, results):
        """Uploads the analysis results to a desired destination.

        :param results: The list containing the meta info (name,
                        content and content-type) of the files which
                        needs to be uploaded.
        :type results: list of ``FileMetaInfo`` tuples
        """
        raise NotImplementedError(
            'Derived classes must implement "upload_results()".',
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

import logging

import boto
from boto.s3.key import Key

from amira.results_uploader import ResultsUploader


class S3Handler(object):
    """Handles the operations with S3, like retrieving the key
    (object) contents from a bucket and creating a new key
    (object) with the contents of a given file.
    AWS and boto use the ambiguous term "key" to describe the objects
    inside the S3 bucket. They are unrelated to AWS keys used to access
    the resources.
    """

    def __init__(self):
        self._s3_connection = boto.connect_s3()

    def get_contents_as_string(self, bucket_name, key_name):
        """Retrieves the S3 key (object) contents.

        :param bucket_name: The S3 bucket name.
        :type bucket_name: string

        :param key_name: The S3 key (object) name.
        :type key_name: string

        :returns: The key (object) contents as a string.
        :rtype: string
        """
        bucket = self._s3_connection.get_bucket(bucket_name, validate=False)
        key = bucket.get_key(key_name)
        contents = key.get_contents_as_string()
        return contents


class S3ResultsUploader(ResultsUploader):
    """Uploads the analysis results to an S3 bucket.

    :param bucket_name: The name of the S3 bucket where the analysis
                        results will be uploaded.
    :type bucket_name: string
    """

    def __init__(self, bucket_name):
        self._bucket_name = bucket_name

        logging.info(
            'Connecting to S3 to obtain access to {0} bucket.'.format(
                bucket_name,
            ),
        )
        s3_connection = boto.connect_s3()
        self._bucket = s3_connection.get_bucket(bucket_name, validate=False)
        logging.info(
            'S3 bucket {0} retrieved successfully.'.format(
                bucket_name,
            ),
        )

    def upload_results(self, results):
        """Uploads the analysis results to an S3 bucket.

        :param results: The list containing the meta info (name,
                        content and content-type) of the files which
                        needs to be uploaded.
        :type results: list of ``FileMetaInfo`` tuples
        """
        for file_meta_info in results:
            logging.info(
                'Uploading the analysis results in the file "{0}" to the S3 '
                'bucket "{1}"'.format(file_meta_info.name, self._bucket_name),
            )
            self._create_object_from_file(file_meta_info)

    def _create_object_from_file(self, file_meta_info):
        """Creates a new key (object) in the S3 bucket with the
        contents of a given file.
        """
        key = Key(self._bucket)
        key.key = file_meta_info.name
        key.set_contents_from_file(
            file_meta_info.content,
            headers={'Content-Type': file_meta_info.content_type},
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Even for a larger incident response team handling all of the repetitive tasks
related to malware infections is a tedious task. Our malware analysts have
spent a lot of time chasing digital forensics from potentially infected macOS
systems, leveraging open source tools, like OSXCollector. Early on, we have
automated some part of the analysis process, augmenting the initial set of
digital forensics collected from the machines with the information gathered
from the threat intelligence APIs. They helped us with additional information
on potentially suspicious domains, URLs and file hashes. But our approach to
the analysis still required a certain degree of configuration and manual
maintenance that was consuming lots of attention from malware responders.

Enter automation: turning all of your repetitive tasks in a scripted way that
will help you deal faster with the incident discovery, forensic collection and
analysis, with fewer possibilities to make a mistake. We went ahead and turned
OSXCollector toolkit into AMIRA: Automated Malware Incident Response and
Analysis service. AMIRA turns the forensic information gathered by OSXCollector
into actionable response plan, suggesting the infection source as well as
suspicious files and domains requiring a closer look. Furthermore, we
integrated AMIRA with our incident response platform, making sure that as
little interaction as necessary is required from the analyst to follow the
investigation. Thanks to that, the incident response team members can focus on
what they excel at: finding unusual patterns and the novel ways that malware
was trying to sneak into the corporate infrastructure.
"""
from __future__ import absolute_import

from setuptools import find_packages
from setuptools import setup

from amira import __version__


with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='amira',
    version=__version__,
    description='Automated Malware Incident Response and Analysis',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Yelp Security',
    author_email='opensource@yelp.com',
    license='The MIT License (MIT)',
    url='https://github.com/Yelp/amira',
    setup_requires='setuptools',
    packages=find_packages(exclude=['tests']),
    provides=['amira'],
    install_requires=[
        'boto',
        'osxcollector_output_filters>=1.1.1',
        'simplejson',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

import logging
import os
import tarfile

try:
    from cStringIO import StringIO as ByteBuffer
    from cStringIO import StringIO as StringBuffer
except ImportError:
    from io import BytesIO as ByteBuffer
    from io import StringIO as StringBuffer

from osxcollector.output_filters.analyze import AnalyzeFilter
from osxcollector.output_filters.base_filters import output_filter

from amira.results_uploader import FileMetaInfo


class DataProcessor(object):

    def __init__(self):
        # List to store processing outputs
        self._results = []

    def process_input(self, tardata):
        """Process input TAR file

        :param tardata: TAR byte stream
        :return: processed data file stream
        """
        raise NotImplementedError()

    def perform_analysis(self, input_stream, data_feeds=None):
        """Perform analysis of forensic input.
        Analysis results should be handled as internal object state

        :param input_stream: forensic data
        :param data_feeds: additional data feeds which may be required in the analysis
        """
        raise NotImplementedError()

    def upload_results(self, file_basename, result_uploaders):
        """Upload forensic results.
        These must be stored as FileMetaInfo objects in the `_results` list attribute

        :param file_basename: Basename used to generate output filenames (prepended)
        :param result_uploaders: List of Uploader objects to invoke
        """
        results = [
            FileMetaInfo(file_basename + res.name, res.content, res.content_type) for res in self._results
            if isinstance(res, FileMetaInfo) and DataProcessor.get_buffer_size(res.content) > 0
        ]
        if results:
            for res_uploader in result_uploaders:
                res_uploader.upload_results(results)
        else:
            logging.warning('No results to upload for {}'.format(file_basename))

    @staticmethod
    def get_buffer_size(data_buffer):
        """Get byte size of file-like object

        :param data_buffer: file-like object
        :return: total size in bytes
        """
        data_buffer.seek(0, os.SEEK_END)
        size = data_buffer.tell()
        data_buffer.seek(0)
        return size


class OSXCollectorDataProcessor(DataProcessor):

    def process_input(self, tardata):
        """Extracts JSON file containing the OSXCollector output from
        tar.gz archive. It will look in the archive contents for the
        file with the extension ".json". If no file with this extension
        is found in the archive or more than one JSON file is found, it
        will raise `OSXCollectorOutputExtractionError`.

        :param tardata: Input TAR archive data
        """
        self._results = [FileMetaInfo('.tar.gz', ByteBuffer(tardata), 'application/gzip')]
        # create a file-like object based on the S3 object contents as string
        fileobj = ByteBuffer(tardata)
        tar = tarfile.open(mode='r:gz', fileobj=fileobj)
        json_tarinfo = [t for t in tar if t.name.endswith('.json')]

        if len(json_tarinfo) != 1:
            raise OSXCollectorOutputExtractionError(
                'Expected 1 JSON file inside the OSXCollector output archive, '
                'but found {0} instead.'.format(len(json_tarinfo)),
            )

        tarinfo = json_tarinfo[0]
        logging.info('Extracted OSXCollector output JSON file {0}'.format(tarinfo.name))
        return tar.extractfile(tarinfo)

    def perform_analysis(self, input_stream, data_feeds=None):
        """Runs Analyze Filter on the OSXCollector output retrieved
        from an S3 bucket.

        :param input_stream: Input data stream on which filters should be ran
        :param data_feeds: black/whitelist data feeds
        """
        analysis_output = StringBuffer()
        text_analysis_summary = ByteBuffer()
        html_analysis_summary = ByteBuffer()

        analyze_filter = AnalyzeFilter(
            monochrome=True,
            text_output_file=text_analysis_summary,
            html_output_file=html_analysis_summary,
            data_feeds=data_feeds or {},
        )

        output_filter._run_filter(
            analyze_filter,
            input_stream=input_stream,
            output_stream=analysis_output,
        )

        # rewind the output files
        analysis_output.seek(0)
        text_analysis_summary.seek(0)
        html_analysis_summary.seek(0)

        self._results += [
            FileMetaInfo('_analysis.json', analysis_output, 'application/json'),
            FileMetaInfo('_summary.txt', text_analysis_summary, 'text/plain'),
            FileMetaInfo('_summary.html', html_analysis_summary, 'text/html; charset=UTF-8'),
        ]


class OSXCollectorOutputExtractionError(Exception):
    """Raised when an unexpected number of JSON files is found in the
    OSXCollector output archive.
    """
    pass
